@@ -1,6 +1,6 @@
 /*
 RealTimeBattle, a robot programming game for Unix
-Copyright (C) 1998-1999  Erik Ouchterlony and Ragnar Ouchterlony
+Copyright (C) 1998-2000  Erik Ouchterlony and Ragnar Ouchterlony
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -172,9 +172,10 @@ ArenaRealTime::parse_arena_file(String& filename)
 
   int succession = 1;
   double scale = the_opts.get_d(OPTION_ARENA_SCALE);
+  double angle_factor = 1.0;
   do
     {
-      parse_arena_line(file, scale, succession);
+      parse_arena_line(file, scale, succession, angle_factor);
 
     } while( !file.eof() );
 
@@ -290,9 +291,14 @@ ArenaRealTime::print_to_logfile(const char first_letter ... )
       break;
 
     case 'L': // List of robot properties
-      LOG_FILE << va_arg(args, int  ) << " ";  // robot id
-      LOG_FILE << hex2str(va_arg(args, long )) << " ";  // robot colour
-      LOG_FILE << va_arg(args, char*);  // robot name
+      {
+        LOG_FILE << va_arg(args, int  ) << " ";  // robot id
+        LOG_FILE << hex2str(va_arg(args, long )) << " ";  // robot colour
+       
+        String name = va_arg(args, char*);   // robot name
+        if( name == "" ) name = "Anonymous";
+        LOG_FILE << name;
+      }
       break;
 
     case 'A': // Arena file line
@@ -350,7 +356,7 @@ ArenaRealTime::broadcast(const message_to_robot_type msg_type ...)
       switch(message_to_robot[msg_type].arg_type[i])
         {
         case NONE: 
-          Error(true, "Couldn't send message, no arg_type", "Robot::send_message");
+          Error(true, "Couldn't send message, no arg_type", "ArenaRealTime::broadcast");
           break;
         case INT:
           str += (String)va_arg(args, int) + ' ';
@@ -365,7 +371,7 @@ ArenaRealTime::broadcast(const message_to_robot_type msg_type ...)
           str += hex2str(va_arg(args, int)) + ' ';
           break;
         default:
-          Error(true, "Couldn't send message, unknown arg_type", "Robot::send_message");
+          Error(true, "Couldn't send message, unknown arg_type", "ArenaRealTime::broadcast");
         }
     }
   str += '\n';
@@ -523,7 +529,8 @@ ArenaRealTime::add_cookie()
       found_space = space_available(pos, r*2.0);
     }
   
-  if( !found_space ) Error(false, "Couldn't find space for cookie", "ArenaRealTime::timeout_function");
+  if( !found_space ) return;
+
   Cookie* cookiep = new Cookie(pos, en);
   object_lists[COOKIE].insert_last( cookiep );
 
@@ -546,7 +553,8 @@ ArenaRealTime::add_mine()
       found_space = space_available(pos, r*2.0);
     }
   
-  if( !found_space ) Error(false, "Couldn't find space for mine", "ArenaRealTime::timeout_function");
+  if( !found_space )  return;
+
   Mine* minep = new Mine(pos, en);
   object_lists[MINE].insert_last( minep );
 
@@ -885,10 +893,18 @@ ArenaRealTime::start_sequence()
   // execute robot processes
 
 
+  Robot* robotp;
   ListIterator<Robot> li;
   for( all_robots_in_sequence.first(li); li.ok(); li++ )
     {
-      li()->start_process();
+      robotp = li();
+      if( robotp->is_networked() )
+        {
+          *robotp->get_outstreamp() << "@R" << endl;
+          robotp->set_values_at_process_start_up();
+        }      
+      else
+        robotp->start_process();
     }
   
   // wait a second before checking
@@ -909,7 +925,8 @@ ArenaRealTime::start_sequence_follow_up()
   for( all_robots_in_sequence.first(li); li.ok(); li++ )
     {
       robotp = li();
-      if( !(robotp->is_process_running()) ) 
+      if( !(robotp->is_networked()) &&
+          !(robotp->is_process_running()) ) 
         {
           all_robots_in_sequence.remove(li);
           robots_left--;
@@ -1008,7 +1025,7 @@ start_tournament(const List<start_tournament_info_t>& robotfilename_list,
     {
       if( the_gui.is_messagewindow_up() )
         MessageWindow::clear_clist( NULL, the_gui.get_messagewindow_p() );
-      else if( !use_message_file )
+      else //if( !use_message_file )
         the_gui.open_messagewindow();
 
       if( !the_gui.is_scorewindow_up() ) the_gui.open_scorewindow();
