@@ -1,6 +1,6 @@
 /*
 RealTimeBattle, a robot programming game for Unix
-Copyright (C) 1998-1999  Erik Ouchterlony and Ragnar Ouchterlony
+Copyright (C) 1998-2000  Erik Ouchterlony and Ragnar Ouchterlony
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -41,14 +41,19 @@ Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 //extern class ArenaRealTime the_arena;
 
-
+//Warning!!! NO COLONS IN LABELS
 
 Options::Options()
 {
 #ifdef HAVE_LOCALE_H
   setlocale( LC_MESSAGES, "" );
+  setlocale( LC_NUMERIC, "POSIX" );
 #endif
+#ifdef RTB_LOCALEDIR
   bindtextdomain( "RealTimeBattle", RTB_LOCALEDIR );
+#else
+  bindtextdomain( "RealTimeBattle", "/usr/local/share/locale" );
+#endif
   textdomain( "RealTimeBattle" );
 
   // Entries are: entry_datatype, page in option, 
@@ -74,6 +79,11 @@ Options::Options()
                           false, true, "Slide/brake friction",
                           _("Slide/brake friction") );
 
+  all_long_options[OPTION_SEND_ROBOT_COORDINATES] =
+    option_info_t<long>(ENTRY_INT, PAGE_ENVIRONMENT, 0, 0, 2, 4,
+                        true, true, "Send robot coordinates", 
+                        _("Send robot coordinates") );
+
   all_double_options[OPTION_ROBOT_MAX_ACCELERATION] = 
     option_info_t<double>(ENTRY_DOUBLE, PAGE_ROBOT, 2.0, 0.1, 10.0, 12,
                           true, true, "Robot max acceleration",
@@ -296,9 +306,14 @@ Options::Options()
 
   all_long_options[OPTION_LOG_EVERY_NTH_UPDATE_INTERVAL] =
     option_info_t<long>(ENTRY_INT, PAGE_TIME, 1, 1, 1000, 5,
-                        false, false, "Logging frequency [Each n:th update interval]",
+                        false, false, "Logging frequency [Each nth update interval]",
                         _("Logging frequency [Each n:th update interval]") );
 
+  all_long_options[OPTION_MAX_ROBOTS_ALLOWED] =
+    option_info_t<long>(ENTRY_INT, PAGE_MISC, 120, 1, 10000, 6, false, false,
+                        "Max robots allowed simultaneously",
+                        _("Max robots allowed simultaneously") );
+
   all_string_options[OPTION_ROBOT_SEARCH_PATH] =
     option_info_t<String>(ENTRY_CHAR, PAGE_MISC, "", "", "", 1000,
                           false, false, "Robot search path", _("Robot search path") );
@@ -419,6 +434,7 @@ Options::broadcast_opts()
   realtime_arena.broadcast( GAME_OPTION, TIMEOUT, get_d(OPTION_TIMEOUT));  
 
   realtime_arena.broadcast( GAME_OPTION, DEBUG_LEVEL, (double)the_arena.get_debug_level());  
+  realtime_arena.broadcast( GAME_OPTION, SEND_ROBOT_COORDINATES, (double)get_l(OPTION_SEND_ROBOT_COORDINATES));  
 }
 
 void
