@@ -1,6 +1,6 @@
 /*
 RealTimeBattle, a robot programming game for Unix
-Copyright (C) 1998-1999  Erik Ouchterlony and Ragnar Ouchterlony
+Copyright (C) 1998-2000  Erik Ouchterlony and Ragnar Ouchterlony
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -78,6 +78,10 @@ Robot::Robot(const String& filename)
   else
     robot_plain_filename = get_segment(robot_filename, nr+1, -1);
 
+
+  network_robot = false;
+
+
   plain_robot_name = "";
   robot_name = "";
   robot_name_uniqueness_number = 0;
@@ -98,8 +102,7 @@ Robot::Robot(const String& filename)
 
   instreamp = NULL;
   outstreamp = NULL;
-  pipes[0] = -1;  
-  pipes[1] = -1;
+  pipes[0] = pipes[1] = -1;
   pid = -1;
   last_drawn_robot_center = Vector2D(infinity,infinity);
 
@@ -214,9 +217,11 @@ Robot::start_process()
           if( robotp != this ) robotp->delete_pipes();
         }
 
-      if( the_arena.get_game_mode() != ArenaBase::DEBUG_MODE )
-        {
-          struct rlimit res_limit;
+      // file access and child process creation are now allowed
+
+      //      if( the_arena.get_game_mode() != ArenaBase::DEBUG_MODE )
+      //        {
+          //          struct rlimit res_limit;
 
           //        // Deny file access
           
@@ -232,17 +237,17 @@ Robot::start_process()
 
           // Forbid creation of child processes
           
-#ifdef HAVE_RLIMIT_NPROC
-          if( getrlimit( RLIMIT_NPROC, &res_limit ) == -1 )
-            Error(true, "Couldn't get proc limits for robot " + robot_filename, 
-                  "Robot::start_process, child");
+//  #ifdef HAVE_RLIMIT_NPROC
+//            if( getrlimit( RLIMIT_NPROC, &res_limit ) == -1 )
+//              Error(true, "Couldn't get proc limits for robot " + robot_filename, 
+//                    "Robot::start_process, child");
           
-          res_limit.rlim_cur = 0;
-          if( setrlimit( RLIMIT_NPROC, &res_limit ) == -1 )
-            Error(true, "Couldn't limit child processes for robot " + robot_filename, 
-                  "Robot::start_process, child");
-#endif
-        }
+//            res_limit.rlim_cur = 0;
+//            if( setrlimit( RLIMIT_NPROC, &res_limit ) == -1 )
+//              Error(true, "Couldn't limit child processes for robot " + robot_filename, 
+//                    "Robot::start_process, child");
+//  #endif
+      //        }
 
       // Execute process. Should not return!
       if( execl(robot_filename.chars(), robot_filename.chars(), NULL) == -1 )
@@ -297,60 +302,101 @@ Robot::is_process_running()
   return process_running;
 }
 
+void
+Robot::update_cpu_time_used()
+{
+  if( network_robot || !process_running ) return;
+
+  String procfilename = "/proc/" + String(pid) + "/stat";
+  ifstream procfile(procfilename.chars());
+  if( !procfile ) return;
+  
+  char buf[16];
+  
+  // step to the correct value
+  for(int i=0; i<15; i++) 
+    procfile >> buf; 
+  
+  // The next four values should be added to get the correct cpu usage
+  double current_cpu = 0.0;
+  int jiffies;  
+  for(int i=0; i<4; i++)
+    {
+      procfile >> jiffies;  
+      current_cpu += (double)jiffies / 100.0;
+    }
+
+//    if( getrusage( pid, &ru ) != 0 ) return;
+
+//    current_cpu = ((double)( ru->ru_utime.tv_usec + ru->ru_stime.tv_usec )) / 1e6 +
+//      (double)( ru->ru_utime.tv_sec + ru->ru_stime.tv_sec );
+
+  cpu_used += current_cpu - last_cpu;
+
+  last_cpu = current_cpu;
+}
+
+void
+Robot::reset_cpu_time()
+{
+  if( network_robot ) return;
+  
+  update_cpu_time_used();
+  cpu_used = 0;
+}
+
 void
 Robot::check_process()
 {
-  String procfilename = "/proc/" + String(pid) + "/stat";
+  //  String procfilename = "/proc/" + String(pid) + "/stat";
 
-  if( is_process_running() )
-    {
-      ifstream procfile(procfilename.chars());
-      if( !procfile ) 
-        {
-          process_running = false;
-          return;
-        }
+  if( !is_process_running() || !alive || network_robot ) return;
 
-      char buf[16];
+    //    ifstream procfile(procfilename.chars());
+//        if( !procfile ) 
+//          {
+//            process_running = false;
+//            return;
+//          }
 
-      for(int i=0; i<13; i++)
-        procfile >> buf;
+//        char buf[16];
+
+//        for(int i=0; i<13; i++)
+//          procfile >> buf;
       
-      int jiffies;
+//        int jiffies;
       
-      procfile >> jiffies;
+//        procfile >> jiffies;
+
+      update_cpu_time_used();
 
-      double current_cpu = (double)jiffies / 100.0;
       double tot_time = the_arena.get_total_time() + time_survived_in_sequence;
-      if( !alive ) tot_time = time_survived_in_sequence;
-
-      if( current_cpu > cpu_next_limit )
+      if( cpu_used > cpu_limit )
         {
-          if( tot_time >= cpu_timeout )
-            {
-              // add time
-              cpu_warning_limit = cpu_next_limit + 
-                the_opts.get_d(OPTION_CPU_EXTRA) * the_opts.get_d(OPTION_CPU_WARNING_PERCENT);
-              cpu_next_limit += the_opts.get_d(OPTION_CPU_EXTRA);
+          if( tot_time >= cpu_timeout ) // ok, within time limit
+            { 
+              // add extra time
+              reset_cpu_time();
+              cpu_limit = the_opts.get_d(OPTION_CPU_EXTRA);
+              cpu_warning_limit = cpu_limit * the_opts.get_d(OPTION_CPU_WARNING_PERCENT);
               cpu_timeout = tot_time + the_opts.get_d(OPTION_CPU_PERIOD);
             }
-          else
-            {
-              // cpu limit exceeded, robot disqualified
+          else // cpu limit exceeded, robot disqualified
+            {              
               die();
-              // add time for next game
-              cpu_warning_limit = cpu_next_limit + 
-                the_opts.get_d(OPTION_CPU_EXTRA) * the_opts.get_d(OPTION_CPU_WARNING_PERCENT);
-              cpu_next_limit += the_opts.get_d(OPTION_CPU_EXTRA);
-              cpu_timeout = tot_time + the_opts.get_d(OPTION_CPU_PERIOD);
+
+              // restart with full cpu_time next game
+              reset_cpu_time(); 
+              cpu_limit = the_opts.get_d(OPTION_CPU_START_LIMIT);
+              cpu_warning_limit = cpu_limit * the_opts.get_d(OPTION_CPU_WARNING_PERCENT);
+              cpu_timeout = 0.0;
             }
         }
-      else if( current_cpu > cpu_warning_limit && tot_time < cpu_timeout )
+      else if( cpu_used > cpu_warning_limit && tot_time < cpu_timeout )
         {
-          send_message( WARNING, PROCESS_TIME_LOW, String(cpu_next_limit - current_cpu).chars());
-          cpu_warning_limit = cpu_next_limit;
+          send_message( WARNING, PROCESS_TIME_LOW, String(cpu_limit - cpu_used).chars());
+          cpu_warning_limit = cpu_limit; // No more warnings
         }
-    }
 }
 
 
@@ -365,13 +411,22 @@ void
 Robot::send_signal()
 {
   if( send_usr_signal )
-    kill(pid, signal_to_send);
+    {
+      if( network_robot )
+        *outstreamp << "@S" << signal_to_send << endl;
+      else if( pid > 0 )
+        kill(pid, signal_to_send);
+    }
 }
 
 void
 Robot::kill_process_forcefully()
 {
-  kill(pid, SIGKILL);
+  if( network_robot )
+    *outstreamp << "@K" << endl;
+  else if( pid > 0 )
+    kill(pid, SIGKILL);
+
   delete_pipes();
   process_running = false;
 }
@@ -379,10 +434,14 @@ Robot::kill_process_forcefully()
 void
 Robot::delete_pipes()
 {
-  if( instreamp != NULL ) delete instreamp;
-  instreamp = NULL;
-  if( outstreamp != NULL ) delete outstreamp;
-  outstreamp = NULL;
+  if( ! network_robot )
+    {
+      if( instreamp != NULL ) delete instreamp;
+      instreamp = NULL;
+      if( outstreamp != NULL ) delete outstreamp;
+      outstreamp = NULL;
+    }
+
   if( pipes[0] != -1 ) 
     {
       close(pipes[0]);
@@ -393,7 +452,6 @@ Robot::delete_pipes()
       close(pipes[1]);
       pipes[1] = -1;
     }
-  
 }
 
 void
@@ -688,7 +746,16 @@ Robot::update_radar_and_cannon(const double timestep)
       double en = ((Robot*)col_obj)->get_energy();
       send_message(ROBOT_INFO, rint( en / lvls ) * lvls, 0);
     }
+
+  if( the_opts.get_l(OPTION_SEND_ROBOT_COORDINATES) == 1 ) // Relative starting pos
+    send_message(COORDINATES, center[0] - start_pos[0], center[1] - start_pos[1], 
+                 robot_angle.pos - start_angle);
+  
+  if( the_opts.get_l(OPTION_SEND_ROBOT_COORDINATES) == 2 ) // Absolute coordinates
+    send_message(COORDINATES, center[0], center[1], robot_angle.pos);
+
   send_message(INFO, the_arena.get_total_time(), length(velocity), cannon_angle.pos); 
+  
 }
 
 void
@@ -717,7 +784,7 @@ Robot::bounce_on_wall(const double bounce_c, const double hardness_c, const Vect
   double injury = en_diff * 0.5 * (h + hardness_c ) * (1.0-e) * (1.0-p);
   change_energy(-injury);
 
-  send_message(COLLISION, WALL, -vec2angle(normal)-robot_angle.pos);
+  send_message(COLLISION, WALL, vec2angle(-normal)-robot_angle.pos);
 }
 
 void
@@ -760,22 +827,25 @@ bounce_on_robot(Robot& robot1, Robot& robot2, const Vector2D& normal)
   robot1.velocity += tmp;
   robot2.velocity -= tmp;
 
-  double an = vec2angle(normal);
+  double an = vec2angle(-normal);
   double en_diff = 0.5 * mass * lengthsqr(start_vel1 - robot1.velocity);
   double injury = en_diff * 0.5 * (h1 + h2) * (1.0-e) * (1.0-p1);
   robot1.change_energy(-injury);
   robot1.send_message(COLLISION, ROBOT, an-robot1.robot_angle.pos);
 
+  an = vec2angle(normal);
   en_diff = 0.5 * mass * lengthsqr(start_vel2 - robot2.velocity);
   injury = en_diff * 0.5 * (h1 + h2) * (1.0-e) * (1.0-p2);
   robot2.change_energy(-injury);
-  robot2.send_message(COLLISION, ROBOT, -an-robot2.robot_angle.pos);
+  robot2.send_message(COLLISION, ROBOT, an-robot2.robot_angle.pos);
 }
 
 void
 Robot::set_values_before_game(const Vector2D& pos, const double angle)
 {
   center = pos;
+  start_pos = pos;
+  start_angle = angle;
   robot_angle.set_rot (angle, 0.0, -infinity, infinity, NORMAL_ROT);
   cannon_angle.set_rot(0.0,   0.0, -infinity, infinity, NORMAL_ROT);
   radar_angle.set_rot (0.0,   0.0, -infinity, infinity, NORMAL_ROT);
@@ -793,15 +863,20 @@ void
 Robot::set_values_at_process_start_up()
 {
   process_running = true;
-  has_saved = false;
+
+  if( !network_robot )
+    {
+      reset_cpu_time();
+      cpu_limit = the_opts.get_d(OPTION_CPU_START_LIMIT);
+      cpu_warning_limit = cpu_limit * the_opts.get_d(OPTION_CPU_WARNING_PERCENT);
+      cpu_timeout = 0.0;
+    }  
+
   time_survived_in_sequence = 0.0;
-  cpu_next_limit = the_opts.get_d(OPTION_CPU_START_LIMIT);
-  cpu_warning_limit = cpu_next_limit * the_opts.get_d(OPTION_CPU_WARNING_PERCENT);
-  cpu_timeout = 0.0;
 
   if( statistics.is_empty() )       // first sequence !
     {
-      send_message(INITIALIZE, 1);  
+      send_message(INITIALIZE, 1);
       colour_given = false;
       name_given = false;
     }
@@ -1024,7 +1099,13 @@ Robot::get_messages()
 
                 case USE_NON_BLOCKING:
                   *instreamp >> val;
-                  set_non_blocking_state( val );                  
+                  if( network_robot )
+                    {
+                      *outstreamp << '@' << ( val ? 'N' : 'B' ) << endl;
+                      send_message(INITIALIZE, 1);  
+                    }
+                  else
+                    set_non_blocking_state( val );
                   break;
 
                 default:
@@ -1311,10 +1392,9 @@ Robot::get_messages()
                        ( center, dir, shot_radius*1.00001, 
                          cl_shape, col_obj, this))    >   radius+1.5*shot_radius )
                     {
-                      //cerr << "Shot has space available after all?" <<  endl;
                       cerr << "dist: " << dist << "      r+1.5sh_r: " << radius+1.5*shot_radius << endl;
-                      cerr << "col_shape: " << cl_shape << endl; 
-                      Error(true, "Shot has space available after all?", "Robot::get_messages");                  
+                      cerr << "col_shape: " << (int)cl_shape << endl; 
+                      Error(false, "Shot has space available after all?", "Robot::get_messages");                  
                     }
                   switch(cl_shape)
                     {
