@@ -1,6 +1,6 @@
 /*
 RealTimeBattle, a robot programming game for Unix
-Copyright (C) 1998-1999  Erik Ouchterlony and Ragnar Ouchterlony
+Copyright (C) 1998-2000  Erik Ouchterlony and Ragnar Ouchterlony
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -24,6 +24,9 @@ Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 #include <gtk/gtk.h>
 
 #if HAVE_DIRENT_H
+# if HAVE_SYS_TYPES_H
+#  include <sys/types.h>
+# endif
 # include <dirent.h>
 # define NAMLEN(dirent) strlen((dirent)->d_name)
 #else
@@ -57,6 +60,8 @@ StartTournamentWindow::StartTournamentWindow( const int default_width,
                                               const int default_x_pos,
                                               const int default_y_pos )
 {
+  tournament_started_flag = false;
+
   filesel = NULL;
 
   // The window widget
@@ -441,7 +446,11 @@ StartTournamentWindow::add_clist( GtkWidget* clist, GtkWidget* box )
   gtk_container_add( GTK_CONTAINER( scrolled_win ),
                      clist );
 
-  GtkStyle* clist_style = gtk_style_new();
+  GtkStyle* clist_style = gtk_rc_get_style(window_p);
+  if( clist_style == NULL )
+    clist_style = gtk_style_new();
+  else
+    clist_style = gtk_style_copy(clist_style);
   clist_style->base[GTK_STATE_NORMAL] = *(the_gui.get_bg_gdk_colour_p());
   clist_style->base[GTK_STATE_ACTIVE] = make_gdk_colour( 0xffffff );
   clist_style->bg[GTK_STATE_SELECTED] = make_gdk_colour( 0xf0d2b4 );
@@ -665,7 +674,7 @@ StartTournamentWindow::save_tournament_file( const String& full_filename,
         max_value = 10000;
       else
         {
-          max_value = min(120,robot_number);
+          max_value = min( the_opts.get_l( OPTION_MAX_ROBOTS_ALLOWED ),robot_number);
         }
       if(i != 1)
         min_value = 1;
@@ -741,8 +750,8 @@ StartTournamentWindow::set_entry( GtkWidget* widget,
           {
             int number_of_robots = mmf_p->stw_p->
               get_selected_robot_tournament()->number_of_elements();
-            if( number_of_robots > 120 )
-              number_of_robots = 120;
+            if( number_of_robots > the_opts.get_l( OPTION_MAX_ROBOTS_ALLOWED ) )
+              number_of_robots = the_opts.get_l( OPTION_MAX_ROBOTS_ALLOWED );
             gtk_entry_set_text
               ( GTK_ENTRY( mmf_p->stw_p->get_entries()[mmf_p->entry] ),
                 String(number_of_robots).chars() );
@@ -760,8 +769,8 @@ StartTournamentWindow::set_entry( GtkWidget* widget,
 
         if( number_of_robots < robots_per_sequence )
           robots_per_sequence = number_of_robots;
-        if( robots_per_sequence > 120 )
-          robots_per_sequence = 120;
+        if( robots_per_sequence > the_opts.get_l( OPTION_MAX_ROBOTS_ALLOWED ) )
+          robots_per_sequence = the_opts.get_l( OPTION_MAX_ROBOTS_ALLOWED );
         if( robots_per_sequence < 2 )
           robots_per_sequence = 2;
         gtk_entry_set_text
@@ -775,8 +784,8 @@ StartTournamentWindow::set_entry( GtkWidget* widget,
         int number_of_arenas = mmf_p->stw_p->
           get_selected_arena_tournament()->number_of_elements();
 
-        if( number_of_arenas > 120 )
-          number_of_arenas = 120;
+        if( number_of_arenas > the_opts.get_l( OPTION_MAX_ROBOTS_ALLOWED ) )
+          number_of_arenas = the_opts.get_l( OPTION_MAX_ROBOTS_ALLOWED );
         if( number_of_arenas < 1 )
           number_of_arenas = 1;
         gtk_entry_set_text
@@ -791,6 +800,9 @@ void
 StartTournamentWindow::start( GtkWidget* widget,
                               class StartTournamentWindow* stw_p )
 {
+  if( stw_p->get_tournament_started_flag() == true )
+    return;
+
   if( the_arena_controller.is_started() )
     if( the_arena.get_state() != NOT_STARTED && the_arena.get_state() != FINISHED )
       return;
@@ -807,7 +819,7 @@ StartTournamentWindow::start( GtkWidget* widget,
         max_value = 10000;
       else
         {
-          max_value = min(120,robot_number);
+          max_value = min(the_opts.get_l( OPTION_MAX_ROBOTS_ALLOWED ),robot_number);
         }
       if(i != 1)
         min_value = 1;
@@ -822,6 +834,7 @@ StartTournamentWindow::start( GtkWidget* widget,
   if( robot_number > 1 &&
       !( stw_p->get_selected_arena_tournament()->is_empty() ) )
     {
+      stw_p->set_tournament_started_flag( true );
       the_arena_controller.start_realtime_arena();
       realtime_arena.start_tournament( *( stw_p->get_selected_robot_tournament() ),
                                        *( stw_p->get_selected_arena_tournament() ),
