@@ -1,6 +1,6 @@
 /*
 RealTimeBattle, a robot programming game for Unix
-Copyright (C) 1998-1999  Erik Ouchterlony and Ragnar Ouchterlony
+Copyright (C) 1998-2000  Erik Ouchterlony and Ragnar Ouchterlony
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -271,7 +271,11 @@ StatisticsWindow::StatisticsWindow( const int default_width,
                       (gpointer) this );
   gtk_container_add( GTK_CONTAINER( scrolled_win ), clist );
 
-  GtkStyle* clist_style = gtk_style_new();
+  GtkStyle* clist_style = gtk_rc_get_style(window_p);
+  if( clist_style == NULL )
+    clist_style = gtk_style_new();
+  else
+    clist_style = gtk_style_copy(clist_style);
   clist_style->base[GTK_STATE_NORMAL] = *(the_gui.get_bg_gdk_colour_p());
   clist_style->base[GTK_STATE_ACTIVE] = make_gdk_colour( 0xffffff );
   clist_style->bg[GTK_STATE_SELECTED] = make_gdk_colour( 0xf0d2b4 );
@@ -388,6 +392,7 @@ StatisticsWindow::change_table_type( GtkWidget* widget,
 #endif
           gtk_clist_set_column_title( GTK_CLIST( clist ), 0, "" );
           gtk_clist_set_column_title( GTK_CLIST( clist ), 1, _("Name") );
+          gtk_clist_column_title_passive( GTK_CLIST( clist ), 0 );
           gtk_clist_set_column_width( GTK_CLIST( clist ), 0, 5 );
           gtk_clist_set_column_width( GTK_CLIST( clist ), 1, 120 );
           gtk_clist_set_column_justification( GTK_CLIST( clist ), 0,
@@ -406,6 +411,7 @@ StatisticsWindow::change_table_type( GtkWidget* widget,
 #endif
           gtk_clist_set_column_title( GTK_CLIST( clist ), 0, _("Seq") );
           gtk_clist_set_column_title( GTK_CLIST( clist ), 1, _("Game") );
+          gtk_clist_column_title_active( GTK_CLIST( clist ), 0 );
           gtk_clist_set_column_width( GTK_CLIST( clist ), 0, 40 );
           gtk_clist_set_column_width( GTK_CLIST( clist ), 1, 45 );
           gtk_clist_set_column_justification( GTK_CLIST( clist ), 0,
