@@ -1,6 +1,6 @@
 /*
 RealTimeBattle, a robot programming game for Unix
-Copyright (C) 1998-1999  Erik Ouchterlony and Ragnar Ouchterlony
+Copyright (C) 1998-2000  Erik Ouchterlony and Ragnar Ouchterlony
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -229,11 +229,18 @@ split_colonseparated_dirs(String& dirs, List<String>& str_list)
 
 bool
 check_if_filename_is_robot( String& fname )
-{
+{ 
   struct stat filestat;
-  if( 0 == stat( fname.chars(), &filestat ) )
-    if( S_ISREG( filestat.st_mode) && (filestat.st_mode & S_IXOTH) ) // Check if file is a regular file that can be executed
-      return true;
+  if( stat( fname.chars(), &filestat ) != 0 ) 
+    return false;
+
+
+  // Check if file is a regular file that can be executed and ends with .robot
+  if( S_ISREG( filestat.st_mode) && 
+      ( filestat.st_mode & ( S_IXOTH | S_IXGRP | S_IXUSR )) &&
+      ( String(".robot") == get_segment(fname, -6, -1) ) )
+    return true;
+
 
   return false;
 }
@@ -245,8 +252,8 @@ check_if_filename_is_arena( String& fname )
   if( 0 == stat( fname.chars(), &filestat ) && fname.get_length() > 6 )
     // Check if file is a regular file that is readable and ends with .arena
     if( S_ISREG( filestat.st_mode) &&
-        (filestat.st_mode & S_IROTH)  &&
-        String(".arena") == get_segment(fname, -6, -1) )
+        ( filestat.st_mode & ( S_IROTH | S_IRGRP | S_IRUSR ) )  &&
+        ( String(".arena") == get_segment(fname, -6, -1) ) )
       return true;
 
   return false;
